@@ -259,28 +259,28 @@ class Repository():
 
     def get_repository_name(self):
         line = self.line.strip()
+        name = line
         if line.startswith("deb cdrom:"):
             name = _("CD-ROM (Installation Disc)")
         else:
-            elements = self.line.split(" ")
-            name = elements[1].replace("deb-src ", "")
-            name = name.replace("deb ", "")
-            if name.startswith("http://") or name.startswith("ftp://"):                    
-                name = name.replace("http://", "")
-                name = name.replace("ftp://", "")
-                try:
-                    parts = name.split("/")
-                    if len(parts) > 0:
-                        name = parts[0]
-                        subparts = name.split(".")
-                        if len(subparts) > 2:
-                            if subparts[-2] != "co":
-                                name = subparts[-2].capitalize()
-                            else:
-                                name = subparts[-3].capitalize()
-                            name = name.replace("Linuxmint", "Linux Mint")
-                except:
-                    pass
+            try:
+                elements = self.line.split(" ")
+                for element in elements:
+                   for protocol in ['http://', 'ftp://', 'https://']:
+                        if element.startswith(protocol):
+                            name = element.replace(protocol, "").split("/")[0]
+                            subparts = name.split(".")
+                            if len(subparts) > 2:
+                                if subparts[-2] != "co":
+                                    name = subparts[-2].capitalize()
+                                else:
+                                    name = subparts[-3].capitalize()
+                            break
+                name = name.replace("Linuxmint", "Linux Mint")
+                name = name.replace("01", "Intel")
+                name = name.replace("Steampowered", "Steam")
+            except:
+                pass
             if self.line.startswith("deb-src"):
                 name = "%s (%s)" % (name, _("Sources")) 
         return "<b>%s</b>\n<small><i>%s</i></small>\n<small><i>%s</i></small>" % (name, self.line, self.file)
@@ -463,7 +463,7 @@ class MirrorSelectionDialog(object):
             represented_speed = ("0 %s") % _("kB/s")
         return represented_speed
     
-    def _speed_test(self, model, iter, depth=0):    
+    def _speed_test(self, model, iter, depth=0): 
         self._speed_test_lock.acquire()
         try:
             if iter is not None:
@@ -485,28 +485,30 @@ class MirrorSelectionDialog(object):
             c.perform()
             download_speed = c.getinfo(pycurl.SPEED_DOWNLOAD) # bytes/sec
         except pycurl.error, error:
-            errno, errstr = error            
+            errno, errstr = error  
+            print "Error '%s' on url %s" % (errstr, url)
+            download_speed = 0
             if errno == 28:
                 # TIMEOUT - but since we're not launching all tests in one go, it doesn't necessarilly mean the server is timeout, maybe we're out of sockets on the client side, so let's retry
-                if (depth < 5):                    
+                if (depth < 3):                    
                     self._speed_test(model, iter, depth+1)
-                else:
-                    # Too many timeouts, removing the mirror..
-                    self._speed_test_lock.acquire()
-                    model.remove(iter)
-                    self._speed_test_lock.release()
-                return
-            else:
-                print "Error '%s' on url %s" % (errstr, url)
-                download_speed = 0
-                # Dodgy mirror, removing it...
-                try:
-                    self._speed_test_lock.acquire()
-                    if iter is not None:
-                        model.remove(iter)
-                    self._speed_test_lock.release()
-                except Exception, detail:
-                    print detail                
+                    return
+                # else:
+                #     # Too many timeouts, removing the mirror..
+                #     self._speed_test_lock.acquire()
+                #     if iter is not None:
+                #         model.remove(iter)
+                #     self._speed_test_lock.release()
+                # return
+            # else:                
+            #     # Dodgy mirror, removing it...
+            #     try:
+            #         self._speed_test_lock.acquire()
+            #         # if iter is not None:
+            #         #     model.remove(iter)
+            #         self._speed_test_lock.release()
+            #     except Exception, detail:
+            #         print detail                
 
         if thread.get_ident() in self._meaningful_speed_threads:  #otherwise, thread is "expired"
             model.set_value(iter, MirrorSelectionDialog.MIRROR_SPEED_COLUMN, download_speed)
@@ -582,45 +584,51 @@ class Application(object):
 
         self.builder.get_object("reload_button_label").set_markup("%s" % _("No action required"))
 
-        self.builder.get_object("label_title_official").set_markup("%s" % _("Official repositories"))     
-        self.builder.get_object("label_title_ppa").set_markup("%s" % _("PPAs"))     
-        self.builder.get_object("label_title_3rd").set_markup("%s" % _("Additional repositories"))     
-        self.builder.get_object("label_title_keys").set_markup("%s" % _("Authentication keys"))     
+        self.builder.get_object("label_title_official").set_markup("%s" % _("Official repositories"))
+        self.builder.get_object("label_title_ppa").set_markup("%s" % _("PPAs"))
+        self.builder.get_object("label_title_3rd").set_markup("%s" % _("Additional repositories"))
+        self.builder.get_object("label_title_keys").set_markup("%s" % _("Authentication keys"))
         self.builder.get_object("label_title_maintenance").set_markup("%s" % _("Maintenance"))
 
-        self.builder.get_object("label_mirrors").set_markup("<b>%s</b>" % _("Mirrors"))    
+        self.builder.get_object("label_mirrors").set_markup("<b>%s</b>" % _("Mirrors"))
         self.builder.get_object("label_mirror_description").set_markup("%s (%s)" % (_("Main"), self.config["general"]["codename"]) )
         self.builder.get_object("label_base_mirror_description").set_markup("%s (%s)" % (_("Base"), self.config["general"]["base_codename"]) )
         self.builder.get_object("button_mirror").set_tooltip_text(_("Select a faster server..."))
         self.builder.get_object("button_base_mirror").set_tooltip_text(_("Select a faster server..."))
 
-        self.builder.get_object("label_optional_components").set_markup("<b>%s</b>" % _("Optional components"))                    
+        self.builder.get_object("label_optional_components").set_markup("<b>%s</b>" % _("Optional components"))
         self.builder.get_object("label_source_code").set_markup("<b>%s</b>" % _("Source code"))
 
         self.builder.get_object("label_ppa_add").set_markup("%s" % _("Add a new PPA..."))
         self.builder.get_object("label_ppa_edit").set_markup("%s" % _("Edit URL..."))
-        self.builder.get_object("label_ppa_remove").set_markup("%s" % _("Remove permanently"))
+        self.builder.get_object("label_ppa_remove").set_markup("%s" % _("Remove"))
+        self.builder.get_object("label_ppa_examine").set_markup("%s" % _("Open PPA"))
+        self.builder.get_object("label_ppa_examine").set_tooltip_text(_("Look inside the PPA and install packages it provides"))
 
         self.builder.get_object("label_repository_add").set_markup("%s" % _("Add a new repository..."))
         self.builder.get_object("label_repository_edit").set_markup("%s" % _("Edit URL..."))
-        self.builder.get_object("label_repository_remove").set_markup("%s" % _("Remove permanently"))
+        self.builder.get_object("label_repository_remove").set_markup("%s" % _("Remove"))
 
         self.builder.get_object("label_keys_add").set_markup("%s" % _("Import key file..."))
         self.builder.get_object("label_keys_fetch").set_markup("%s" % _("Download a key..."))
-        self.builder.get_object("label_keys_remove").set_markup("%s" % _("Remove permanently"))
+        self.builder.get_object("label_keys_remove").set_markup("%s" % _("Remove"))
 
         self.builder.get_object("button_mergelist_label").set_markup("%s" % _("Fix MergeList problems"))
         self.builder.get_object("button_mergelist").set_tooltip_text("%s" % _("If you experience MergeList problems, click this button to solve the problem."))
         self.builder.get_object("button_purge_label").set_markup("%s" % _("Purge residual configuration"))
         self.builder.get_object("button_purge").set_tooltip_text("%s" % _("Packages sometimes leave configuration files on the system even after they are removed."))
-        
+        self.builder.get_object("button_remove_foreign_label").set_markup("%s" % _("Remove foreign packages"))
+        self.builder.get_object("button_remove_foreign").set_tooltip_text("%s" % _("Packages which do not come from known repositories are listed here and can be removed."))
+        self.builder.get_object("button_downgrade_foreign_label").set_markup("%s" % _("Downgrade foreign packages"))
+        self.builder.get_object("button_downgrade_foreign").set_tooltip_text("%s" % _("Packages which version does not come from known repositories are listed here and can be downgraded."))
+
         self.builder.get_object("label_description").set_markup("<b>%s</b>" % self.config["general"]["description"])
         self.builder.get_object("image_icon").set_from_file("/usr/share/mintsources/%s/icon.png" % self.lsb_codename)
 
         self.builder.get_object("source_code_cb").set_label(_("Enable source code repositories"))
 
         self.builder.get_object("source_code_cb").connect("toggled", self.apply_official_sources)
-               
+
         self.selected_components = []
         if (len(self.optional_components) > 0):
             if os.path.exists("/etc/linuxmint/info"):
@@ -683,6 +691,9 @@ class Application(object):
         self._ppa_treeview = self.builder.get_object("treeview_ppa")
         self._ppa_treeview.set_model(self._ppa_model)
         self._ppa_treeview.set_headers_clickable(True)
+        self._ppa_treeview.connect("row-activated", self.on_ppa_treeview_doubleclick)
+        selection = self._ppa_treeview.get_selection()
+        selection.connect("changed", self.ppa_selected)
         
         self._ppa_model.set_sort_column_id(2, gtk.SORT_ASCENDING)
 
@@ -773,6 +784,8 @@ class Application(object):
         self.builder.get_object("button_ppa_add").connect("clicked", self.add_ppa)
         self.builder.get_object("button_ppa_edit").connect("clicked", self.edit_ppa)
         self.builder.get_object("button_ppa_remove").connect("clicked", self.remove_ppa)
+        self.builder.get_object("button_ppa_examine").connect("clicked", self.examine_ppa)
+        self.builder.get_object("button_ppa_examine").set_sensitive(False)
 
         self.builder.get_object("button_repository_add").connect("clicked", self.add_repository)
         self.builder.get_object("button_repository_edit").connect("clicked", self.edit_repository)
@@ -784,6 +797,8 @@ class Application(object):
 
         self.builder.get_object("button_mergelist").connect("clicked", self.fix_mergelist)
         self.builder.get_object("button_purge").connect("clicked", self.fix_purge)
+        self.builder.get_object("button_remove_foreign").connect("clicked", self.remove_foreign)
+        self.builder.get_object("button_downgrade_foreign").connect("clicked", self.downgrade_foreign)
         
         # From now on, we handle modifications to the settings and save them when they happen
         self._interface_loaded = True
@@ -809,11 +824,17 @@ class Application(object):
                             mirror_list.append(mirror)
         return mirror_list
 
+    def remove_foreign(self, widget):
+        os.system("/usr/lib/linuxmint/mintSources/foreign_packages.py remove %s" % self._main_window.window.xid)
+
+    def downgrade_foreign(self, widget):
+        os.system("/usr/lib/linuxmint/mintSources/foreign_packages.py downgrade %s" % self._main_window.window.xid)
+
     def fix_purge(self, widget):
         os.system("aptitude purge ~c -y")
         image = gtk.Image()
         image.set_from_file("/usr/lib/linuxmint/mintSources/maintenance.png")
-        self.show_confirmation_dialog(self._main_window, _("There is no more residual configuration on the system."), image, affirmation=True)        
+        self.show_confirmation_dialog(self._main_window, _("There is no more residual configuration on the system."), image, affirmation=True)
 
     def fix_mergelist(self, widget):
         os.system("rm /var/lib/apt/lists/* -vf")
@@ -881,8 +902,14 @@ class Application(object):
     def add_ppa(self, widget):
         image = gtk.Image()
         image.set_from_file("/usr/lib/linuxmint/mintSources/ppa.png")
+        start_line = ""
+        clipboard_text = self.get_clipboard_text("ppa")
+        if clipboard_text != None:
+            start_line = clipboard_text
+        else:
+            start_line = "ppa:username/ppa"
 
-        line = self.show_entry_dialog(self._main_window, _("Please enter the name of the PPA you want to add:"), "ppa:username/ppa", image)
+        line = self.show_entry_dialog(self._main_window, _("Please enter the name of the PPA you want to add:"), start_line, image)
         if line is not None:
             user, sep, ppa_name = line.split(":")[1].partition("/")
             ppa_name = ppa_name or "ppa"
@@ -948,11 +975,55 @@ class Application(object):
                 repository.delete()
                 self.ppas.remove(repository)
 
+    def ppa_selected(self, selection):
+        try:
+            self.builder.get_object("button_ppa_examine").set_sensitive(False)
+            (model, iter) = selection.get_selected()
+            if (iter != None):
+                repository = model.get_value(iter, 0)
+                ppa_name = model.get_value(iter, 2)
+                if repository.line.startswith("deb http://ppa.launchpad.net"):
+                    self.builder.get_object("button_ppa_examine").set_sensitive(True)
+        except Exception, detail:
+            print detail
+
+    def on_ppa_treeview_doubleclick(self, treeview, path, column):
+        self.examine_ppa(None)
+
+    def examine_ppa(self, widget):
+        try:
+            selection = self._ppa_treeview.get_selection()
+            (model, iter) = selection.get_selected()
+            if (iter != None):
+                repository = model.get_value(iter, 0)
+                ppa_name = model.get_value(iter, 2)
+                if repository.line.startswith("deb http://ppa.launchpad.net"):
+                    line = repository.line.split()[1].replace("http://ppa.launchpad.net/", "")
+                    if line.endswith("/ubuntu"):
+                        line = line[:-7]
+                        ppa_owner, ppa_name = line.split("/")
+                        architecture = commands.getoutput("dpkg --print-architecture")
+                        codename = commands.getoutput("lsb_release -u -c -s")
+                        ppa_file = "/var/lib/apt/lists/ppa.launchpad.net_%s_%s_ubuntu_dists_%s_main_binary-%s_Packages" % (ppa_owner, ppa_name, codename, architecture)
+                        if os.path.exists(ppa_file):
+                            os.system("/usr/lib/linuxmint/mintSources/ppa_browser.py %s %s %s" % (ppa_owner, ppa_name, self._main_window.window.xid))
+                        else:
+                            print "%s not found!" % ppa_file
+                            self.show_error_dialog(self._main_window, _("The content of this PPA is not available. Please refresh the cache and try again."))
+        except Exception, detail:
+            print detail
+
     def add_repository(self, widget):
         image = gtk.Image()
         image.set_from_file("/usr/lib/linuxmint/mintSources/3rd.png")
+        start_line = ""
+        clipboard_text = self.get_clipboard_text("deb")
+        if clipboard_text != None:
+            start_line = clipboard_text
+        else:
+            start_line = "deb http://packages.domain.com/ %s main" % self.config["general"]["base_codename"]
 
-        line = self.show_entry_dialog(self._main_window, _("Please enter the name of the repository you want to add:"), "deb http://packages.domain.com/ %s main" % self.config["general"]["base_codename"], image)
+        line = self.show_entry_dialog(self._main_window, _("Please enter the name of the repository you want to add:"), start_line, image)
         if line is not None and line.strip().startswith("deb"):
             # Add the repository in sources.list.d
             with open("/etc/apt/sources.list.d/additional-repositories.list", "a") as text_file:
@@ -1053,7 +1124,7 @@ class Application(object):
         d = gtk.MessageDialog(parent,
                               gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                               gtk.MESSAGE_ERROR,
-                              gtk.BUTTONS_OK_CANCEL,
+                              gtk.BUTTONS_OK,
                               message)
 
         d.set_markup(message)
@@ -1288,6 +1359,14 @@ class Application(object):
             if url in selected_base_mirror:
                 if os.path.exists("/usr/lib/linuxmint/mintSources/flags/%s.png" % mirror.country_code.lower()):
                     self.builder.get_object("image_base_mirror").set_from_file("/usr/lib/linuxmint/mintSources/flags/%s.png" % mirror.country_code.lower()) 
+                  
+    def get_clipboard_text(self, source_type):
+        clipboard = gtk.Clipboard(display=gtk.gdk.display_get_default(), selection="CLIPBOARD")
+        text = clipboard.wait_for_text()
+        if text is not None and text.strip().startswith(source_type):
+            return text
+        else:
+            return None
 
 if __name__ == "__main__":
     if os.getuid() != 0:
